"""Unit test for result."""

import unittest

from e2fyi.utils.core.results import Result


class ResultHelper(unittest.TestCase):
    """TestCase for Result"""

    def test_ok(self):
        expected_value = "foo bar"
        result_ok = Result[str](expected_value)
        self.assertTrue(result_ok.is_ok)
        self.assertEqual(result_ok.value, expected_value)
        self.assertEqual(result_ok.with_default("bar foo"), expected_value)

    def test_not_ok_with_default(self):
        default_value = "foo bar"
        expected_exception = ValueError("some value error.")

        result_exception_and_value = Result[str](
            "bar foo", exception=expected_exception
        )
        self.assertFalse(result_exception_and_value.is_ok)
        self.assertEqual(
            result_exception_and_value.with_default(default_value), default_value
        )
        self.assertEqual(result_exception_and_value.exception, expected_exception)

        result_exception_only = Result[str](exception=expected_exception)
        self.assertFalse(result_exception_only.is_ok)
        self.assertEqual(
            result_exception_only.with_default(default_value), default_value
        )
        self.assertEqual(result_exception_only.exception, expected_exception)



"""utils to interact with s3 buckets."""
import io
import json
import logging
import os.path

from typing import IO, Dict, List, Union, Generic, TypeVar, Callable, Optional

import boto3
import magic
import pandas as pd
import botocore.exceptions

from pydantic import BaseModel

from e2fyi.utils.core.results import Result

T = TypeVar("T")
StringOrBytes = TypeVar("StringOrBytes", bytes, str)


def _noop(key: T) -> T:
    """Do nothing"""
    return key


class S3Resource(Generic[StringOrBytes]):
    """
    S3Resource is a wrapper class to bind the relationship between a S3 object
    and its local representation (e.g. local file, in-memory stream).
    """

    def __init__(
        self,
        filename: str,
        content_type: str,
        bucketname: str = "",
        prefix: str = "",
        protocol: str = "s3a://",
        stream: Union[io.StringIO, io.BytesIO, IO[StringOrBytes]] = None,
        metadata: Dict[str, str] = None,
    ):
        """
        Creates a new instance of S3Resource.

        Args:
            filename (str): filename of the object.
            content_type (str): mime type of the object.
            bucketname (str, optional): name of the bucket the obj is or should be.
                Defaults to "".
            prefix (str, optional): prefix to be added to the filename to get the s3
                object key. Defaults to "".
            protocol (str, optional): s3 client protocol. Defaults to "s3a://".
            stream (Union[io.StringIO, io.BytesIO, IO[StringOrBytes]], optional): data
                stream. Defaults to None.
            metadata (dict, optional): metadata for the object. Defaults to None.
        """
        self.filename = filename
        self.content_type = content_type
        self.bucketname = bucketname
        self.prefix = prefix
        self.protocol = protocol
        self.stream: Optional[
            Union[io.StringIO, io.BytesIO, IO[StringOrBytes]]
        ] = stream
        self.metadata = metadata or {}

    @property
    def key(self) -> str:
        """Key for the resource."""
        if not self.filename:
            raise ValueError("filename cannot be empty.")
        return "%s%s" % (self.prefix, self.filename)

    @property
    def uri(self) -> str:
        """URI to the resource."""
        if not self.bucketname:
            raise ValueError("bucketname cannot be empty.")
        return "%s%s/%s" % (self.protocol, self.bucketname, self.key)

    def read(self, size=-1) -> StringOrBytes:
        """duck-typing for a readable stream."""
        if self.stream:
            return self.stream.read(size)  # type: ignore
        raise RuntimeError(
            "Unable to read from stream: S3Resource does not have a stream."
        )

    def seek(self, offset: int, whence: int = 0) -> int:
        """duck-typing for readable stream.
        See https://docs.python.org/3/library/io.html

        Change the stream position to the given byte offset. offset is interpreted
        relative to the position indicated by whence. The default value for whence
        is SEEK_SET. Values for whence are:

            SEEK_SET or 0 – start of the stream (the default); offset should be zero
                or positive

            SEEK_CUR or 1 – current stream position; offset may be negative

            SEEK_END or 2 – end of the stream; offset is usually negative

        Return the new absolute position.
        """
        if self.stream:
            return self.stream.seek(offset, whence)  # type: ignore
        raise RuntimeError(
            "Unable to seek from stream: S3Resource does not have a stream."
        )

    def close(self) -> "S3Resource":
        """Close the resource stream."""
        if self.stream:
            self.stream.close()
        return self

    def get_value(self) -> StringOrBytes:
        """Retrieve the entire contents of the S3Resource."""
        self.seek(0)
        return self.read()

    def load(
        self, constructor: Callable[..., T] = None, unpack: bool = True
    ) -> Union[dict, list, T]:
        """
        load the content of the stream into memory using `json.loads`. If a
        `constructor` is provided, it will be used to create a new object. Setting
        `unpack` to be true will unpack the content when creating the object
        with the `constructor` (i.e. * for list, ** for dict)

        Args:
            constructor (Callable[..., T], optional): A constructor function.
                Defaults to None.
            unpack (bool, optional): whether to unpack the content when passing
                it to the constructor. Defaults to True.

        Raises:
            TypeError: [description]

        Returns:
            Union[dict, list, T]: [description]
        """
        if not self.content_type == "application/json":
            raise TypeError("Content type is not 'application/json'.")

        self.seek(0)  # reset buffer offset
        result = json.loads(self.read())

        if not constructor:
            return result  # type: ignore
        if unpack:
            if isinstance(result, dict):
                return constructor(**result)
            if isinstance(result, list):
                return constructor(*result)
        return constructor(result)

    def __str__(self) -> str:
        """String representation of a S3Resource."""
        try:
            return self.uri
        except ValueError:
            return "%s(metadata=%s)" % (self.__class__.__name__, self.metadata)


class S3ResourceHelper:
    """
    S3ResourceHelper is a helper class to wrap S3Resource over local files or
    in-memory data.

    Example::

        import pandas as pd

        from e2fyi.utils.aws import S3ResourceHelper
        from pydantic import BaseModel

        # creates a text s3 resource
        text_resource = S3ResourceHelper.wrap("hello world", "text/plain")

        # creates a json s3 resource
        json_resource = S3ResourceHelper.wrap({"hello": "world"}, "application/json")

        # creates a s3 resource from file
        file_resource = S3ResourceHelper.wrap(open("some_file.csv"), "text/csv")

        # creates a s3 resource from a dataframe or series
        df = pd.DataFrame([{"key": "a", "value": 1}, {"key": "b", "value": 2}])
        df_csv_resource = S3ResourceHelper.wrap(df, "text/csv", index=False)
        df_json_resource = S3ResourceHelper.wrap(df,
                                                 "application/json",
                                                 orient="records")


        # creates a s3 resource from a pydantic basemodel
        class DummyModel(BaseModel):
            key: string
            value: int

        model = DummyModel(key="a", value=1)
        model_resource = S3ResourceHelper.wrap(model, "application/json")
    """

    @classmethod
    def wrap(
        cls,
        obj: Union[
            str, bytes, dict, IO[StringOrBytes], pd.DataFrame, pd.Series, BaseModel
        ],
        content_type: str,
        metadata: dict = None,
        **kwargs
    ) -> S3Resource:
        """
        wrap returns a S3Resource with a data stream of the provided obj.

        Args:
            obj (Union[str, bytes, dict, IO[StringOrBytes], pd.DataFrame, pd.Series,
                BaseModel]): data
            content_type (str): mime type of data
            metadata (dict, optional): Additional metadata for s3 object. Defaults to
                None.

        Raises:
            TypeError: "Cannot create S3Resource because obj type is not supported: %s"

        Returns:
            S3Resource: S3Resource with a data stream of the provided obj.
        """

        if isinstance(obj, (str, bytes, dict)):
            return cls.wrap_raw(obj, content_type, metadata=metadata)

        if isinstance(obj, (pd.DataFrame, pd.Series)):
            if content_type == "application/json":
                return cls.wrap_pandas_as_json(obj, metadata=metadata, **kwargs)
            return cls.wrap_pandas_as_csv(obj, metadata=metadata, **kwargs)

        if isinstance(obj, BaseModel):
            return cls.wrap_raw(obj.dict(), "application/json", metadata=metadata)

        if hasattr(obj, "read"):
            return cls.wrap_io(obj, content_type, metadata=metadata)

        raise TypeError(
            "Cannot create S3Resource because obj type is not supported: %s" % type(obj)
        )

    @classmethod
    def wrap_raw(
        cls, obj: Union[str, bytes, dict], content_type: str, metadata: dict = None
    ) -> S3Resource:
        """
        wrap_raw returns a S3Resource with a corresponding io stream.

        Args:
            obj (Union[str, bytes, dict]): data
            content_type (str): mime type for the data
            metadata (dict, optional): Additional metadata for s3 object. Defaults to
                None.

        Raises:
            ValueError: "obj must be one of the following: string, bytes, or dict."

        Returns:
            S3Resource: S3Resource with a corresponding io stream.
        """
        if isinstance(obj, str):
            if os.path.isfile(obj):
                return cls.wrap_file(obj, content_type, metadata=metadata)
            return cls.wrap_io(io.StringIO(obj), content_type, metadata=metadata)

        if isinstance(obj, bytes):
            return cls.wrap_io(io.BytesIO(obj), content_type, metadata=metadata)

        if isinstance(obj, dict):
            stream = io.StringIO(json.dumps(obj))
            return cls.wrap_io(stream, "application/json", metadata=metadata)

        raise ValueError("obj must be one of the following: string, bytes, or dict.")

    @staticmethod
    def wrap_pandas_as_csv(
        df: Union[pd.DataFrame, pd.Series], metadata: dict = None, **kwargs: dict
    ) -> S3Resource:
        """
        wrap_pandas_as_csv returns a S3Resource with a csv stream.

        Args:
            df (Union[pd.DataFrame, pd.Series]): pandas dataframe
            metadata (dict, optional): Additional metadata for s3 object. Defaults to
                None.
            **kwargs: keyword arguments to pass to pandas.to_csv method.

        Returns:
            S3Resource: S3Resource with a csv stream.
        """
        stream = io.StringIO()
        df.to_csv(stream, **kwargs)
        # set buffer position to beginning as there should not be any write
        # operation after this.
        stream.seek(0)
        return S3Resource[StringOrBytes](
            filename="",
            content_type="application/csv",
            stream=stream,
            metadata={"source": "%s" % type(df), **(metadata or {})},
        )

    @staticmethod
    def wrap_pandas_as_json(
        df: Union[pd.DataFrame, pd.Series], metadata: dict = None, **kwargs: dict
    ) -> S3Resource:
        """
        wrap_pandas_as_json returns a S3Resource with a json stream.

        Args:
            df (Union[pd.DataFrame, pd.Series]): pandas dataframe
            metadata (dict, optional): Additional metadata for s3 object. Defaults to
                None.
            **kwargs: keyword arguments to pass to pandas.to_json method.

        Returns:
            S3Resource: S3Resource with a json stream.
        """
        stream = io.StringIO()
        df.to_json(stream, **kwargs)
        # set buffer position to beginning as there should not be any write
        # operation after this.
        stream.seek(0)
        return S3Resource[StringOrBytes](
            filename="",
            content_type="application/json",
            stream=stream,
            metadata={"source": "%s" % type(df), **(metadata or {})},
        )

    @staticmethod
    def wrap_io(
        stream: IO[StringOrBytes], content_type: str, metadata: dict = None
    ) -> S3Resource:
        """
        wrap_io wraps returns a S3Resource with the io stream.

        Args:
            stream (IO): io stream
            content_type (str): mime type for the data
            metadata (dict, optional): Additional metadata for s3 object. Defaults to
                None.

        Returns:
            S3Resource: S3Resource with the io stream.
        """

        return S3Resource[StringOrBytes](
            filename="",
            content_type=content_type,
            stream=stream,
            metadata={"source": "%s" % type(stream), **(metadata or {})},
        )

    @staticmethod
    def wrap_file(
        filepath: str, content_type: str = "", metadata: dict = None
    ) -> S3Resource:
        """
        wrap_file returns a S3Resource with a binary data stream.
        If content_type is not provided, method will attempt to infer from the
        file.

        Args:
            filepath (str): path to the file.
            content_type (str, optional): mime type. Defaults to "".
            metadata (dict, optional): Additional metadata for s3 object. Defaults to
                None.

        Returns:
            S3Resource: [description]
        """
        content_type = (
            content_type if content_type else magic.from_file(filepath, mime=True)
        )
        prefix = os.path.dirname(filepath) or ""
        filename = os.path.basename(filepath)

        return S3Resource[bytes](
            filename=filename,
            content_type=content_type,
            prefix=prefix,
            stream=open(filepath, "rb"),
            metadata={"source": filepath, **(metadata or {})},
        )


class S3Bucket:
    """
    S3Bucket models a s3 bucket configuration.

    Example::

        from e2fyi.utils.aws import S3Bucket

        # upload a dict to s3 bucket
        S3Bucket("foo").upload("some_folder/some_file.json", {"foo": "bar"})

        # creates a s3 bucket with std prefix rule
        foo_bucket = S3Bucket("foo",
                              get_prefix=lambda prefix: "some_folder/%s" % prefix)
        foo_bucket.upload("some_file.json", {"foo": "bar"})

        # list files
        S3Bucket("foo").list("some_folder/")

        # list files inside "some_folder/"
        foo_bucket.list()

    """

    def __init__(
        self,
        name: str,
        get_prefix: Callable[[str], str] = _noop,
        s3client: boto3.client = None,
    ):
        """
        Creates a new instance of s3 bucket.

        Args:
            name (str): name of the bucket
            get_prefix (Callable[[str], str], optional): function that takes a filename
                and return the full path to the resource in the bucket.
            s3client (boto3.Session.client, optional): use a custom boto3
                s3 client.
        """
        self.name = name
        self.prefix = get_prefix("")
        self._get_prefix = get_prefix
        self._s3client = s3client or boto3.client("s3")

    def __str__(self) -> str:
        return self.name

    def upload(
        self,
        filepath: str,
        body: Union[
            IO[StringOrBytes],
            str,
            bytes,
            dict,
            BaseModel,
            pd.DataFrame,
            pd.Series,
            S3Resource[StringOrBytes],
        ],
        content_type: str = "text/plain",
        protocol: str = "s3a://",
        metadata: dict = None,
        **kwargs
    ) -> Result[S3Resource[StringOrBytes]]:
        """Upload a payload to s3 bucket."""

        key = self._get_prefix(filepath)
        try:
            resource = S3ResourceHelper.wrap(body, content_type, **kwargs)
            resource.bucketname = self.name
            resource.protocol = protocol
            resource.prefix = (
                os.path.join(resource.prefix, os.path.dirname(key))
                if resource.prefix
                else os.path.dirname(key) + os.path.sep
            )
            resource.filename = os.path.basename(key)
            if metadata:
                resource.metadata.update(metadata)

            self._s3client.upload_fileobj(
                resource,
                Bucket=resource.bucketname,
                Key=resource.key,
                ExtraArgs={
                    "ContentType": resource.content_type,
                    "Metadata": resource.metadata or {},
                },
                Callback=lambda n: logging.info(
                    "[%s/%s] bytes transferred: %d",
                    resource.bucketname,
                    resource.key,
                    n,
                ),
            )
        except (botocore.exceptions.ClientError, ValueError) as exc:
            return Result(None, exception=exc)

        return Result(resource)

    def list(
        self, prefix: str = "", max_keys: int = 255, within_project: bool = True
    ) -> Result[List[str]]:
        """Get a list of keys in an S3 bucket."""
        # constraint list to within project
        if within_project:
            prefix = self._get_prefix(prefix)
        try:
            resp = boto3.client("s3").list_objects_v2(
                Bucket=self.name, Prefix=prefix, MaxKeys=max_keys
            )
            items = [obj["Key"] for obj in resp["Contents"]]
        except botocore.exceptions.ClientError as exc:
            return Result(None, exception=exc)
        return Result(items)



"""
Helpers for interacting with S3 buckets.

S3 bucket with preset prefix::

    from e2fyi.utils.aws import S3Bucket

    # upload a dict to s3 bucket
    S3Bucket("foo").upload("some_folder/some_file.json", {"foo": "bar"})

    # creates a s3 bucket with std prefix rule
    foo_bucket = S3Bucket("foo", get_prefix=lambda prefix: "some_folder/%s" % prefix)
    foo_bucket.upload("some_file.json", {"foo": "bar"})  # some_folder/some_file.json

Uploading to S3 bucket::

    import logging

    import pandas as pd

    from e2fyi.utils.aws import S3Bucket
    from pydantic import BaseModel

    # s3 bucket with prefix rule
    s3 = S3Bucket("foo", get_prefix=lambda prefix: "some_folder/%s" % prefix)

    # check if upload is successful
    result = s3.upload("some_file.txt", "hello world")
    if not result.is_ok:
        logging.exception(result.exception)

    # upload string as text/plain file
    s3.upload("some_file.txt", "hello world")

    # upload dict as application/json file
    s3.upload("some_file.json", {"foo": "bar"})

    # upload pandas df as text/csv and/or application/json files
    df = pd.DataFrame([{"key": "a", "value": 1}, {"key": "b", "value": 2}])
    # extra kwargs can be passed to pandas.to_csv method
    s3.upload("some_file.csv", df, content_type="text/csv", index=False)
    # extra kwargs can be passed to pandas.to_json method
    s3.upload("some_file.json", df, content_type="application/json", orient="records")

    # upload pydantic models as application/json file
    class KeyValue(BaseModel):
        key: str
        value: int
    model = KeyValue(key="a", value=1)
    s3.upload("some_file.json", model)


Listing contents inside S3 buckets::

    from e2fyi.utils.aws import S3Bucket

    # list files
    S3Bucket("foo").list("some_folder/")

    # list files inside "some_folder/"
    foo_bucket.list()
"""
from e2fyi.utils.aws.s3 import S3Bucket, S3Resource, S3ResourceHelper



"""This module provides a Result generic class to describe an unreliable output.
"""
import json

from typing import Generic, TypeVar, Optional

T = TypeVar("T")


class Result(Generic[T]):
    """
    Result is a generic class used to describe an output that can potentially
    fails (e.g. raise exception).


    Example::

        import logging

        from e2fyi.utils.core import Result


        def load_from_file(filepath: str) -> Result[string]:
            try:
                with open(filepath, "r") as fp:
                    return Result(fp.read())
            except IOError as err:
                return Result(exception=err)

        data = load_from_file("some_file.json")

        # print with a default value fallback
        print(data.with_default("default value"))

        # print data if ok, else log exception
        if data.is_ok:
            print(data)
        else:
            logging.exception(data.exception)
    """

    def __init__(self, value: Optional[T] = None, exception: BaseException = None):
        """Creates a new instance of Result. If an exception is provided, the
        Result value is considered to be not ok."""
        self.value = value
        self.exception = exception

    @property
    def is_ok(self) -> bool:
        """whether the output is generated successfully."""
        return not self.exception

    def with_default(self, default_value: T) -> T:
        """returns a default value if the outputs is not generated successfully."""
        return (self.value or default_value) if self.is_ok else default_value

    def __str__(self) -> str:
        """string representation"""
        if self.value and isinstance(self.value, list):
            return "\n".join(self.value)
        if self.value and isinstance(self.value, dict):
            return json.dumps(self.value, indent=2)
        return "%s" % (self.value or self.exception)



"""
Core helpers.


Wrapping function with Result::

    import logging

    from e2fyi.utils.core import Result


    def load_from_file(filepath: str) -> Result[string]:
        try:
            with open(filepath, "r") as fp:
                return Result(fp.read())
        except IOError as err:
            return Result(exception=err)

    data = load_from_file("some_file.json")

    # print with a default value fallback
    print(data.with_default("default value"))

    # print data if ok, else log exception
    if data.is_ok:
        print(data)
    else:
        logging.exception(data.exception)

"""
from e2fyi.utils.core.results import Result



#!/usr/bin/env python
# noqa
# pylint: skip-file
"""The setup script."""
from setuptools import setup

with open("requirements.txt", "r") as filein:
    requirements = filein.readlines()

with open("requirements-dev.txt", "r") as filein:
    test_requirements = filein.readlines()

with open("version.txt", "r") as filein:
    version = filein.read()

with open("README.md", "r") as filein:
    readme = filein.read()


setup_requirements: list = [
    "setuptools >= 41.0.0",
    # python3 specifically requires wheel 0.26
    'wheel; python_version < "3"',
    'wheel >= 0.26; python_version >= "3"',
]

setup(
    author="eterna2",
    author_email="eterna2@hotmail.com",
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Intended Audience :: Developers",
        "Natural Language :: English",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
    ],
    description="General utils for interacting with aws resources.",
    long_description=readme,
    long_description_content_type="text/markdown",
    url="https://github.com/e2fyi/py-utils",
    include_package_data=True,
    package_data={
        "": [
            "version.txt",
            "requirements.txt",
            "requirements-dev.txt",
            "test.py",
            "README.md",
        ]
    },
    keywords="util aws s3 boto3 upload",
    name="e2fyi-utils",
    packages=["e2fyi.utils", "e2fyi.utils.aws", "e2fyi.utils.core", "e2fyi.utils.io"],
    setup_requires=setup_requirements,
    python_requires=">=3.6",
    install_requires=requirements,
    test_suite="e2fyi",
    tests_require=test_requirements,
    version=version,
    zip_safe=False,
)



"""Unit test for s3 helpers."""
import io
import json
import unittest

from unittest.mock import MagicMock

import boto3
import pandas as pd

from pydantic import BaseModel

from e2fyi.utils.aws.s3 import S3Bucket, S3Resource, S3ResourceHelper


class S3ResourceTest(unittest.TestCase):
    """TestCase for S3Resource"""

    def test_basic_ok(self):
        data = {"key1": "foo", "key2": "bar"}
        data_str = json.dumps(data)
        resource = S3Resource(
            "filename.ext",
            content_type="application/json",
            prefix="prefix/",
            bucketname="bucketname",
            protocol="protocol://",
            stream=io.StringIO(data_str),
            metadata={"tag": "metadata"},
        )

        self.assertEqual(resource.key, "prefix/filename.ext")
        self.assertEqual(resource.uri, "protocol://bucketname/prefix/filename.ext")
        self.assertEqual(resource.read(), data_str)
        self.assertDictEqual(resource.load(), data)

    def test_load_dict(self):
        data = {"key1": "foo", "key2": "bar"}
        data_str = json.dumps(data)
        resource = S3Resource(
            "filename.ext",
            content_type="application/json",
            prefix="prefix/",
            bucketname="bucketname",
            protocol="protocol://",
            stream=io.StringIO(data_str),
            metadata={"tag": "metadata"},
        )

        # pass in a transform func and do not unpack content
        transformed = resource.load(
            lambda content: {**content, "extra": True}, unpack=False
        )
        self.assertDictEqual(transformed, {"key1": "foo", "key2": "bar", "extra": True})

        # pass in a basemodel and unpack content
        class DummyClass(BaseModel):
            """dummy class"""

            key1: str
            key2: str

        dummy = resource.load(DummyClass)
        self.assertIsInstance(dummy, DummyClass)
        self.assertDictEqual(dummy.dict(), data)

    def test_load_list(self):
        data = ["a", "b", "c"]
        data_str = json.dumps(data)
        resource = S3Resource(
            "filename.ext",
            content_type="application/json",
            stream=io.StringIO(data_str),
        )

        # pass in a transform func and do not unpack content
        transformed = resource.load(lambda content: content + ["d"], unpack=False)
        self.assertListEqual(transformed, ["a", "b", "c", "d"])
        # pass in a transform func and unpack content
        transformed = resource.load(lambda a, b, c: "%s:%s:%s" % (a, b, c), unpack=True)
        self.assertEqual(transformed, "a:b:c")


class S3ResourceHelperTest(unittest.TestCase):
    """TestCase for S3ResourceHelper"""

    def test_wrap_str(self):
        data = "foo bar"
        resource = S3ResourceHelper.wrap_raw(data, content_type="text/plain")
        self.assertEqual(resource.content_type, "text/plain")
        self.assertEqual(resource.read(), data)

    def test_wrap_dict(self):
        data = {"foo": "bar"}
        resource = S3ResourceHelper.wrap_raw(data, content_type="text/plain")
        self.assertEqual(resource.content_type, "application/json")
        self.assertEqual(json.loads(resource.read()), data)

    def test_pandas_csv(self):
        data = pd.DataFrame(data=[{"name": "a"}, {"name": "b"}])
        resource = S3ResourceHelper.wrap_pandas_as_csv(data, index=False)
        self.assertEqual(resource.content_type, "application/csv")
        self.assertEqual(resource.read(), "name\na\nb\n")

    def test_pandas_dict(self):
        data = pd.DataFrame(data=[{"name": "a"}, {"name": "b"}])
        resource = S3ResourceHelper.wrap_pandas_as_json(data, orient="records")
        self.assertEqual(resource.content_type, "application/json")
        self.assertEqual(resource.load(), [{"name": "a"}, {"name": "b"}])


class S3BucketTest(unittest.TestCase):
    """TestCase for S3Bucket"""

    def setUp(self):
        s3client = boto3.client("s3")
        s3client.upload_fileobj = MagicMock()  # type: ignore
        self.s3client = s3client

    def test_basic(self):
        bucket = S3Bucket(
            "bucketname", get_prefix=lambda filename: "prefix/%s" % filename
        )
        self.assertEqual(bucket.name, "bucketname")
        self.assertEqual(bucket.prefix, "prefix/")

    def test_upload_str(self):
        bucket = S3Bucket(
            "bucketname",
            get_prefix=lambda filename: "prefix/%s" % filename,
            s3client=self.s3client,
        )
        result = bucket.upload("some/path/file.ext", body="hello")
        resource = result.value

        self.s3client.upload_fileobj.assert_called_once()
        self.assertTrue(result.is_ok)

        self.assertIsInstance(resource, S3Resource)
        self.assertEqual(resource.uri, "s3a://bucketname/prefix/some/path/file.ext")
        self.assertEqual(resource.content_type, "text/plain")
        self.assertEqual(resource.read(), "hello")

    def test_upload_dict(self):
        bucket = S3Bucket(
            "bucketname",
            get_prefix=lambda filename: "prefix/%s" % filename,
            s3client=self.s3client,
        )

        result = bucket.upload("some/path/file.ext", body={"foo": "bar"})
        resource = result.value

        self.s3client.upload_fileobj.assert_called_once()
        self.assertTrue(result.is_ok)

        self.assertIsInstance(resource, S3Resource)
        self.assertEqual(resource.uri, "s3a://bucketname/prefix/some/path/file.ext")
        self.assertEqual(resource.content_type, "application/json")
        self.assertDictEqual(resource.load(), {"foo": "bar"})

    def test_upload_basemodel(self):
        bucket = S3Bucket(
            "bucketname",
            get_prefix=lambda filename: "prefix/%s" % filename,
            s3client=self.s3client,
        )

        class DummyModel(BaseModel):
            """Dummy base model"""

            some_int: int
            some_str: str

        model = DummyModel(some_int=1, some_str="foo bar")
        result = bucket.upload("some/path/file.ext", body=model)
        resource = result.value

        self.s3client.upload_fileobj.assert_called_once()
        self.assertTrue(result.is_ok)

        self.assertIsInstance(resource, S3Resource)
        self.assertEqual(resource.uri, "s3a://bucketname/prefix/some/path/file.ext")
        self.assertEqual(resource.content_type, "application/json")
        self.assertDictEqual(resource.load(), model.dict())

    def test_upload_pandasdf(self):
        bucket = S3Bucket(
            "bucketname",
            get_prefix=lambda filename: "prefix/%s" % filename,
            s3client=self.s3client,
        )

        df = pd.DataFrame([{"col1": value, "col2": "foo"} for value in range(0, 5)])
        result = bucket.upload(
            "some/path/file.ext", body=df, content_type="application/csv", index=False
        )
        resource = result.value

        self.s3client.upload_fileobj.assert_called_once()
        self.assertTrue(result.is_ok)

        self.assertIsInstance(resource, S3Resource)
        self.assertEqual(resource.uri, "s3a://bucketname/prefix/some/path/file.ext")
        self.assertEqual(resource.content_type, "application/csv")
        self.assertEqual(
            resource.read(), "col1,col2\n0,foo\n1,foo\n2,foo\n3,foo\n4,foo\n"
        )

    def test_upload_pandasdf_asrecord(self):
        bucket = S3Bucket(
            "bucketname",
            get_prefix=lambda filename: "prefix/%s" % filename,
            s3client=self.s3client,
        )

        df = pd.DataFrame([{"col1": value, "col2": "foo"} for value in range(0, 5)])
        result = bucket.upload(
            "some/path/file.ext",
            body=df,
            content_type="application/json",
            orient="records",
        )
        resource = result.value

        self.s3client.upload_fileobj.assert_called_once()
        self.assertTrue(result.is_ok)

        self.assertIsInstance(resource, S3Resource)
        self.assertEqual(resource.uri, "s3a://bucketname/prefix/some/path/file.ext")
        self.assertEqual(resource.content_type, "application/json")
        self.assertListEqual(
            resource.load(),
            [
                {"col1": 0, "col2": "foo"},
                {"col1": 1, "col2": "foo"},
                {"col1": 2, "col2": "foo"},
                {"col1": 3, "col2": "foo"},
                {"col1": 4, "col2": "foo"},
            ],
        )



#!/usr/bin/env python
# noqa
# pylint: skip-file
import sys
import unittest
import subprocess

import coverage


def suite():
    """test suite"""
    test_loader = unittest.TestLoader()
    test_suite = test_loader.discover("e2fyi", pattern="*_test.py")
    return test_suite


def run_unittest():
    cov = coverage.Coverage()
    cov.start()
    print("unittest:")
    unittest.TextTestRunner(verbosity=2).run(suite())
    cov.stop()
    cov.save()

    print("coverage report:")
    cov.report()


def run_pylint():
    print("checking with pylint:")
    subprocess.call(["pylint", "e2fyi"], stdout=sys.stdout, stderr=sys.stderr)


def run_mypy():
    print("checking with mypy:")
    subprocess.call(["mypy", "e2fyi"], stdout=sys.stdout, stderr=sys.stderr)


def run_black():
    print("checking with black:")
    subprocess.call(["black", "--check", "e2fyi"], stdout=sys.stdout, stderr=sys.stderr)


def main():
    run_pylint()
    run_black()
    run_mypy()
    run_unittest()


if __name__ == "__main__":
    main()
